#include "WidgetFactory.h"
#include <map>
#include <cstring>

namespace 
{
	struct MyCompare
	{
		bool operator()(const char* a, const char* b) const
		{
			return std::strcmp(a, b) < 0;
		}
	};

	typedef std::map< const char*, Widget::Creator, MyCompare > CreatorMap;

	CreatorMap& instance()
	{
		static CreatorMap registry;
		return registry;
	}
}

void WidgetFactory::debugDump()
{
	CreatorMap::const_iterator it = instance().begin();
	CreatorMap::const_iterator end = instance().end();

	for( ; it != end; ++it )
	{
		std::printf("%s: 0x%08x\n", it->first, (uintptr_t)(void*)it->second);
	}
}

Widget* WidgetFactory::create(const char* name)
{
	CreatorMap& registry = instance();
	CreatorMap::iterator it = registry.find(name);
	if( registry.end() == it )
		return 0;
	return it->second();
}

void WidgetFactory::add(const char* name, Widget::Creator creator)
{
	instance().insert(CreatorMap::value_type(name, creator));
}
